pub fn conditional_select_nospec_lt(
x: usize,
y: usize,
a: usize,
b: usize,
) -> usizeExpand description
returns a if x < y, b otherwise.
Immune to speculative execution information leak bugs such as Spectre V1.
pub fn conditional_select_nospec_lt(
x: usize,
y: usize,
a: usize,
b: usize,
) -> usizereturns a if x < y, b otherwise.
Immune to speculative execution information leak bugs such as Spectre V1.