pub fn confine_array_index(index: usize, size: usize) -> usizeExpand description
returns index if index < size, or 0 if index >= size.
Immune to speculative execution information leak bugs such as Spectre V1.
pub fn confine_array_index(index: usize, size: usize) -> usizereturns index if index < size, or 0 if index >= size.
Immune to speculative execution information leak bugs such as Spectre V1.