remove_from_container

Function remove_from_container 

pub unsafe fn remove_from_container<T, Tag, P>(obj: &T) -> Option<P>
where P: PtrTraits<Target = T>, T: DoublyLinkedListContainable<T, Tag>,
Removes an object from its container without a reference to the container.

Safety

The caller must ensure that obj is currently in a valid list instance that does NOT track its size (uses NonTrackingSize), and that no other mutable references to that list are active.