Canary

Struct Canary 

pub struct Canary<const MAGIC: u32> { /* private fields */ }
An embeddable structure guard.

To use fbl::Canary, choose a 4-byte guard value.

You can use the Canary::new() method to instantiate a Canary object. The compiler will infer the magic value from the type definition.

struct MyStruct {
    canary: fbl::Canary<{ fbl::magic(b"guar") }>,
    // ...
}

impl MyStruct {
    fn new() -> Self {
        MyStruct {
            canary: fbl::Canary::new(),
            // ...
        }
    }
}

The canary initializes itself with the guard value during construction and checks it during destruction (on Drop). You can also manually check the value during the lifetime of your object by calling the assert method.

If the value is not an ASCII string, you can directly use an integer literal as the const generic parameter.

struct MyStruct {
    canary: fbl::Canary<0x12345678>,
    // ...
}

Implementations

Source

impl<const MAGIC: u32> Canary<MAGIC>

pub const fn new() -> Self

Create a new Canary with the specified magic value.

pub fn assert(&self)

Assert that the value of magic is as expected.

Panics

Panics if self.magic is not the expected value.

pub fn valid(&self) -> bool

Some places have special handling of bad magic values. For these cases, simply return whether the magic is correct, and let them respond appropriately if not.