Cursor

Struct Cursor 

pub struct Cursor<'a, K, P, Tag = DefaultObjectTag, S = NonTrackingSize, O = DefaultWavlTreeObserver<<P as PtrTraits>::Target>>
where P: PtrTraits, P::Target: WavlTreeContainable<P::Target, Tag> + WavlTreeKeyable<K>, K: Ord, S: SizeTracker, O: WavlTreeObserver<Target = P::Target>,
{ /* private fields */ }
A read-only cursor positioned in a WavlTree.

Implementations

Source

impl<'a, K, P, Tag, S, O> Cursor<'a, K, P, Tag, S, O>
where P: PtrTraits, P::Target: WavlTreeContainable<P::Target, Tag> + WavlTreeKeyable<K>, K: Ord, S: SizeTracker, O: WavlTreeObserver<Target = P::Target>,

pub fn get(&self) -> Option<&'a P::Target>

Returns a reference to the current element, or None if the cursor is at the sentinel.

pub fn is_valid(&self) -> bool

Returns true if the cursor is positioned at a valid element (not the sentinel).

pub fn left(&self) -> Self

Returns a cursor positioned at the left child of the current element. If the current element is the sentinel, returns a cursor at the sentinel.

pub fn right(&self) -> Self

Returns a cursor positioned at the right child of the current element.

pub fn parent(&self) -> Self

Returns a cursor positioned at the parent of the current element.

pub fn as_raw_ptr(&self) -> *mut P::Target

Returns the raw pointer to the current element. This may be a sentinel pointer if the cursor is at the sentinel.