Name

Struct Name 

pub struct Name<const SIZE: usize> { /* private fields */ }
A fixed-size, thread-safe name buffer with automatic truncation and null-termination.

Names include the trailing null byte as part of their SIZE-sized buffer. Constructors and setters automatically truncate inputs to SIZE - 1 bytes to ensure the string is always null-terminated.

Corresponds to fbl::Name<Size> in C++.

Implementations

Source

impl<const SIZE: usize> Name<SIZE>

pub fn project<'__pin>( self: Pin<&'__pin mut Self>, ) -> NameProjection<'__pin, SIZE>

Pin-projects all fields of Self.

These fields are structurally pinned:

  • lock

These fields are not structurally pinned:

  • name
Source

impl<const SIZE: usize> Name<SIZE>

pub fn lock_lock(&self) -> impl PinInit<NameLockGuard<'_, SIZE>, Infallible>

pub fn lock_lock_policy<P: LockPolicy<RawSpinlock, AcquireArgs = ()>>( &self, ) -> impl PinInit<NameLockGuard<'_, SIZE, P>, Infallible>

pub fn lock_lock_aliased<'a, AliasClass: LockClass, M2: RawLock>( &'a self, alias: &'a KMutex<AliasClass, M2>, ) -> impl PinInit<KMutexAliasedGuard<'a, NameLockClass, AliasClass, RawSpinlock, M2>, Infallible>

pub fn lock_lock_aliased_policy<'a, AliasClass: LockClass, M2: RawLock, P: LockPolicy<RawSpinlock, AcquireArgs = ()>>( &'a self, alias: &'a KMutex<AliasClass, M2>, ) -> impl PinInit<KMutexAliasedGuard<'a, NameLockClass, AliasClass, RawSpinlock, M2, P>, Infallible>

pub fn guard_lock<'b, 'a>( &'b self, token: &'b LockToken<'a, NameLockClass>, ) -> NameLockTokenGuard<'b, 'a, SIZE>

pub fn guard_lock_mut<'b, 'a>( &'b self, token: &'b mut LockToken<'a, NameLockClass>, ) -> NameLockTokenGuardMut<'b, 'a, SIZE>

Source

impl<const SIZE: usize> Name<SIZE>

pub fn init() -> impl PinInit<Self, Infallible>

Creates a PinInit initializer for Name initialized to an empty string.

Initializes storage directly in-place without stack copies.

pub fn len(&self) -> usize

Returns the length of the string (excluding the null terminator).

pub fn is_empty(&self) -> bool

Returns true if the string is empty.

pub fn get(&self, out_name: &mut [u8])

Copies the name out into out_name.

out_name is zeroed first. If out_name is non-empty, up to min(out_name.len() - 1, SIZE - 1) characters are copied up to the first null byte. If out_name is empty, no data is written.

pub fn set(&self, name: &[u8])

Resets the Name to the given data under lock.

Any characters after the first null byte are ignored. If name is longer than SIZE - 1, it will be truncated to ensure null-termination. The remainder of the internal buffer is filled with null bytes.

pub fn set_str(&self, name: &str)

Resets the Name to the given string slice under lock.

pub fn copy_name(&self) -> [u8; SIZE]

Copies the internal name into a newly stack-allocated buffer of size SIZE.

pub fn copy_from(&self, other: &Self)

Copies the contents of other into self.