RefCounted

Struct RefCounted 

pub struct RefCounted { /* private fields */ }
Support for intrusive atomic reference counting.

This supports intrusive atomic reference counting with adoption. This means that a new object starts life at a reference count of 1 and has to be adopted by a type (such as a fbl::RefPtr) that begins manipulation of the reference count. If the reference count ever reaches zero, the object’s lifetime is over and it should be destroyed (Release() returns true if this is the case).

Implementations

Source

impl RefCounted

pub const fn new() -> Self

Create a new RefCounted.

In debug builds, the initial count is set to a sentinel value to detect use before adoption. In release builds, it starts at 1.

pub fn ref_count_debug(&self) -> i32

Current ref count. Only to be used for debugging purposes.