RefPtr

Struct RefPtr 

pub struct RefPtr<T>{ /* private fields */ }
RefPtr<T> holds a reference to an intrusively-refcounted object of type T that deletes the object when the refcount drops to 0.

T should be a struct that contains a fbl::RefCounted field and implements HasRefCount and Destroy traits.

Implementations

Source

impl<T: HasRefCount + Recyclable> RefPtr<T>

pub unsafe fn from_raw(ptr: *const T) -> Self

Constructs a RefPtr from a raw pointer that has already been adopted.

Safety
  • The caller must ensure that ptr is valid and has a ref count already acquired.
  • ptr must have been allocated in such a way that calling T::recycle(ptr) is a correct way to deallocate the pointer.
pub unsafe fn try_new(value: T) -> Result<RefPtr<T>, AllocError>

Helper function that allocates a new instance of T using T::allocate and returns a RefPtr wrapping it.

This is an internal helper function that should not be used directly. Use the make_ref_counted!(...) macro instead of this function to properly initialize the ref count.

Safety

The caller must ensure that T has a RefCounted field that is not already adopted.

pub fn as_ptr(this: &Self) -> *const T

Returns the raw pointer to the object.

pub fn ptr_eq(a: &Self, b: &Self) -> bool

Returns true if the two RefPtrs point to the same object.

pub fn into_raw(this: Self) -> *const T

Consume the RefPtr and return the raw pointer without modifying the ref count.

The caller is responsible for maintaining the reference count.

pub fn try_pin_init<E>(init: impl PinInit<T, E>) -> Result<Self, E>

Use the given pin-initializer to pin-initialize a T inside of a new RefPtr.

pub fn try_init<E>(init: impl Init<T, E>) -> Result<Self, E>

Use the given initializer to in-place initialize a T inside of a new RefPtr.

pub fn pin_init(init: impl PinInit<T, Infallible>) -> Result<Self, AllocError>

Use the given pin-initializer to pin-initialize a T inside of a new RefPtr.

pub fn init(init: impl Init<T, Infallible>) -> Result<Self, AllocError>

Use the given initializer to in-place initialize a T inside of a new RefPtr.