pub struct RingBuffer<T, const N: usize> { /* private fields */ }Expand description
RingBuffer is a statically-allocated, typed ring buffer container.
This container is not thread safe.
Implementations
Sourceimpl<T, const N: usize> RingBuffer<T, N>
impl<T, const N: usize> RingBuffer<T, N>
Sourcepub fn front_mut(&mut self) -> &mut T
pub fn front_mut(&mut self) -> &mut T
Returns a mutable reference to the oldest element in the buffer.
Panics
Panics if the buffer is empty.
Sourcepub fn back_mut(&mut self) -> &mut T
pub fn back_mut(&mut self) -> &mut T
Returns a mutable reference to the newest element in the buffer.
Panics
Panics if the buffer is empty.