#[repr(C)]pub struct SinglyLinkedListNode<T> {
pub next: UnsafeCell<*mut T>,
}Expand description
A node in a singly linked list.
Fields
next: UnsafeCell<*mut T>The next element in the list. This is null if the node is not in a container. This is a sentinel value (1) if the node is the last element of the list.
Implementations
Sourceimpl<T> SinglyLinkedListNode<T>
impl<T> SinglyLinkedListNode<T>
Sourcepub fn in_container(&self) -> bool
pub fn in_container(&self) -> bool
Returns true if the node is currently in a list.