UniquePtr

Struct UniquePtr 

pub struct UniquePtr<T>
where T: Recyclable,
{ /* private fields */ }
UniquePtr<T> holds a unique pointer to an object of type T and deletes the object when the UniquePtr goes out of scope.

T must implement the Recyclable trait to define how the object is destroyed. This allows UniquePtr to manage both Rust-allocated objects and C++-allocated objects.

Implementations

Source

impl<T: Recyclable> UniquePtr<T>

pub unsafe fn from_raw(ptr: *mut T) -> Self

Constructs a UniquePtr from a raw pointer.

Safety
  • ptr must be valid and the sole owning reference to the object pointed to by ptr.
  • ptr must have been allocated in such a way that calling T::recycle(ptr) is a correct way to deallocate the pointer.
pub fn try_new(value: T) -> Result<UniquePtr<T>, AllocError>

Helper function that allocates a new instance of T using T::allocate and returns a UniquePtr wrapping it.

For Rust-allocated objects where T derives Recyclable, this will use Box::try_new.

pub fn as_ptr(this: &Self) -> *const T

Returns the raw pointer to the object.

pub fn as_mut_ptr(this: &mut Self) -> *mut T

Returns a mutable raw pointer to the object.

pub fn into_raw(this: Self) -> *mut T

Consume the UniquePtr and return the raw pointer without destroying the object.

The caller is responsible for managing the object’s lifetime after this call.