WavlTreeNode

Struct WavlTreeNode 

#[repr(C)]
pub struct WavlTreeNode<T, R: WavlTreeRank = bool> { pub parent: UnsafeCell<*mut T>, pub left: UnsafeCell<*mut T>, pub right: UnsafeCell<*mut T>, pub rank: UnsafeCell<R>, }
A node in a Weak AVL (WAVL) Tree.

Fields

parent: UnsafeCell<*mut T>

The parent element in the tree.

left: UnsafeCell<*mut T>

The left child element in the tree.

right: UnsafeCell<*mut T>

The right child element in the tree.

rank: UnsafeCell<R>

The integer rank of this node.

Implementations

Source

impl<T, R: WavlTreeRank> WavlTreeNode<T, R>

pub const fn new() -> Self

Creates a new, unlinked node.

pub fn in_container(&self) -> bool

Returns true if the node is currently in a tree.

pub fn rank(&self) -> R

Returns the rank value of this node.

pub fn is_valid(&self) -> bool

Returns true if the node state invariants are currently valid.