SinglyLinkedListContainable

Trait SinglyLinkedListContainable 

pub trait SinglyLinkedListContainable<T, Tag = DefaultObjectTag> {
    // Required method
    fn get_node(&self) -> &SinglyLinkedListNode<T>;
}
Trait that types must implement to be contained in a SinglyLinkedList.

The Tag parameter is used to support objects participating in multiple lists simultaneously. By implementing this trait multiple times with different tags, an object can provide different SinglyLinkedListNode instances for each list it belongs to.

Required Methods

fn get_node(&self) -> &SinglyLinkedListNode<T>

Returns a reference to the list node.

Implementors