pub struct DefaultObjectTag;Expand description
The default tag used when an object participates in only one list.
Tags are used to disambiguate which node state should be used when an object participates in multiple intrusive containers simultaneously. By providing a default tag, we make the common case (participating in a single list) more ergonomic by not requiring the user to specify a tag explicitly.