RingBuffer

Struct RingBuffer 

pub struct RingBuffer<T, const N: usize> { /* private fields */ }
RingBuffer is a statically-allocated, typed ring buffer container. This container is not thread safe.

Implementations

Source

impl<T, const N: usize> RingBuffer<T, N>

pub const fn new() -> Self

Create a new, empty RingBuffer.

pub fn size(&self) -> u32

Returns the number of elements in the buffer.

pub const fn capacity() -> usize

Returns the capacity of the buffer.

pub fn is_empty(&self) -> bool

Returns true if the buffer is empty.

pub fn is_full(&self) -> bool

Returns true if the buffer is full.

pub fn front(&self) -> &T

Returns a reference to the oldest element in the buffer.

Panics

Panics if the buffer is empty.

pub fn front_mut(&mut self) -> &mut T

Returns a mutable reference to the oldest element in the buffer.

Panics

Panics if the buffer is empty.

pub fn back(&self) -> &T

Returns a reference to the newest element in the buffer.

Panics

Panics if the buffer is empty.

pub fn back_mut(&mut self) -> &mut T

Returns a mutable reference to the newest element in the buffer.

Panics

Panics if the buffer is empty.

pub fn pop(&mut self)

Removes the oldest element from the buffer.

Panics

Panics if the buffer is empty.

pub fn push(&mut self, obj: T)

Pushes a new element into the buffer.

Panics

Panics if the buffer is full.

pub fn clear(&mut self)

Removes all elements from the buffer.