SlabOrigin

Struct SlabOrigin 

pub struct SlabOrigin<T, L: RawLock = RawMutex, const SLAB_SIZE: usize = DEFAULT_SLAB_ALLOCATOR_SLAB_SIZE, const TRACK_OBJECT_COUNT: bool = false> { /* private fields */ }
A helper type to store the originating slab allocator for instanced allocations.

This type wraps Option<NonNull<SlabAllocator<...>>> and provides safe Send and Sync implementations, allowing the containing object to be shared across threads.

Implementations

Source

impl<T, L: RawLock, const SLAB_SIZE: usize, const TRACK_OBJECT_COUNT: bool> SlabOrigin<T, L, SLAB_SIZE, TRACK_OBJECT_COUNT>

pub const fn new() -> Self

Creates a new, uninitialized SlabOrigin.

pub fn set( &mut self, origin: NonNull<SlabAllocator<T, L, SLAB_SIZE, TRACK_OBJECT_COUNT>>, )

Sets the origin allocator.

pub fn get( &self, ) -> Option<NonNull<SlabAllocator<T, L, SLAB_SIZE, TRACK_OBJECT_COUNT>>>

Gets the origin allocator.